# Copyright (c) 2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Core module.
Defines base aspect class
"""

__revision__ = '$Id: core.py,v 1.7 2003/06/16 15:23:39 adim Exp $'


import types

class AbstractAspect(object):
    """ Abstract Class for all aspects
    """
    
    def __init__(self, cls, method_name):
        """
        cls : the class or class instance to aspect
        method_name : the name of the method to wrap
        raises a RuntimeError if no 
        """
        try:
            self.method = getattr(cls, method_name)
            self.method_name = method_name
            self.weaved_obj = cls
            self.method_dict = self.method.im_func.func_globals
        except AttributeError:
            if type(cls) == types.ClassType:
                class_name = cls.__name__
            else:
                class_name = cls.__class__.__name__            
            raise AttributeError, "Unable to find %s in %s"%\
                  (method_name, class_name)
        
    def before(self, wobj, *args, **kwargs):
        """Define Here what you want to do before the method call
        """
        pass

    def after(self, wobj, ret_v, exec_excpt, *args, **kwargs):
        """Define Here what you want to do after the method call
        """
        pass

    def around(self, wobj, *args, **kwargs):
        """Define Here what you want to do around the method call
        around MUST call PROCEED !!
        """
        return self._proceed(wobj, *args, **kwargs)
        

    def _proceed(self, wobj, *args, **kwargs):
        """The real call
        """
        # XXX FIXME : ugly code, and not bugsafe !
        return self.method.im_func(wobj, *args, **kwargs)
        ## if self.method.im_self is not None:
##             return self.method(*args, **kwargs)
##         else:
##             return self.method(wobj, *args, **kwargs)

    
    def get_wrapped_method(self):
        """Returns the original method
        """
        return self.method

        

    def __str__(self):
        return self.__class__.__name__



