# Copyright (c) 2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This module defines utilities functions to manage functions' prototypes.
"""

__revision__ = '$Id: prototypes.py,v 1.7 2003/06/16 15:23:39 adim Exp $'

import types
from copy import deepcopy

## XXX FIXME : Ne faire qu'une seule fonction pour les
## deux reassign puisqu'ils font exactement la mme chose
def reassign_function_arguments(method, varargs, kwargs):
    """This methods will analyze the prototype of method.

    To wrap a method easily, the most common way to do is
    to declare a simple prototype : def wrapper(self, *args, **kwargs)
    which can match any possible call.
    The problem is that you can't use variable_names originally passed
    in argument of the wrapped function/method.
    Suppose you have : 'def f(arg1, arg2)', your wrapper will be
    'def f_wrapper(*args, **kwargs)' and you then can neither use 'arg1'
    and 'arg2' in the scope of f_wrapper.
    reassign_function_arguments builds a dict (a kind of locals()) with
    the names of the args of the original function in KEYS, and the values
    with which they were called in VALUES.

    returns this dict. (You might use this function like this :

        def a_wrapper(self, *args, **kwargs):
            locals().update(reassign_function_arguments(ori_method,args,kwargs))
            ...

    """

    code = method.func_code
    assignment_dict = {}
    var_offset = 0

    ## XXX FIXME : regarder pourquoi ce test ne marche pas.
    ## Du coup, cette mthode ne marche pas pour les fonctions,
    ## uniquement pour les mthodes.

    if type(method) == types.MethodType:
        # if it's a method => skip 'self' from the variable arg list
        var_offset = 1

    # Get all arg names (without defaults / keywords)
    arg_list = deepcopy(code.co_varnames[var_offset:code.co_argcount])


    # Build the tuple of default values : will hold all the values
    # (in the right order) of all values which haven't been passed to
    # the method.

    # If the method has default arg values
    if method.func_defaults is not None:
        # Take the N right most values in func's default values
        # where N is the number of unspecified variables in the
        # method call.
        default_index = len(arg_list) - len(varargs)
        if default_index:
            default_values = method.func_defaults[-default_index:]
        else:
            default_values = ()
    # Else : no default values for the method args
    else:
        default_values = ()
    
    # Reassign all values to their variables
    # len(arg_list) = len(varargs+default_values)
    index = 0
    for val in varargs+default_values:
        assignment_dict[arg_list[index]] = val
        index += 1

    # Reassign keywords values if necessary
    for var, val in kwargs.items():
        assignment_dict[var] = val

    # Return dict
    return assignment_dict

